/*     */ package org.hibernate.cfg;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collections;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Properties;
/*     */ import java.util.Set;
/*     */ import java.util.StringTokenizer;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.annotations.AnyMetaDef;
/*     */ import org.hibernate.annotations.AnyMetaDefs;
/*     */ import org.hibernate.annotations.MetaValue;
/*     */ import org.hibernate.annotations.common.reflection.XAnnotatedElement;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XPackage;
/*     */ import org.hibernate.annotations.common.reflection.XProperty;
/*     */ import org.hibernate.cfg.annotations.EntityBinder;
/*     */ import org.hibernate.cfg.annotations.Nullability;
/*     */ import org.hibernate.cfg.annotations.TableBinder;
/*     */ import org.hibernate.id.MultipleHiLoPerTableGenerator;
/*     */ import org.hibernate.mapping.Any;
/*     */ import org.hibernate.mapping.Collection;
/*     */ import org.hibernate.mapping.Component;
/*     */ import org.hibernate.mapping.IdGenerator;
/*     */ import org.hibernate.mapping.Join;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.Property;
/*     */ import org.hibernate.mapping.SimpleValue;
/*     */ import org.hibernate.mapping.Table;
/*     */ import org.hibernate.mapping.ToOne;
/*     */ import org.hibernate.mapping.Value;
/*     */ import org.hibernate.type.DiscriminatorType;
/*     */ import org.hibernate.type.Type;
/*     */ import org.hibernate.type.TypeFactory;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BinderHelper
/*     */ {
/*     */   public static final String ANNOTATION_STRING_DEFAULT = "";
/*  52 */   private static Log log = LogFactory.getLog(BinderHelper.class);
/*     */   
/*     */   public static final Set<String> PRIMITIVE_NAMES;
/*     */   
/*     */   static
/*     */   {
/*  58 */     Set<String> primitiveNames = new HashSet();
/*  59 */     primitiveNames.add(Byte.TYPE.getName());
/*  60 */     primitiveNames.add(Short.TYPE.getName());
/*  61 */     primitiveNames.add(Integer.TYPE.getName());
/*  62 */     primitiveNames.add(Long.TYPE.getName());
/*  63 */     primitiveNames.add(Float.TYPE.getName());
/*  64 */     primitiveNames.add(Double.TYPE.getName());
/*  65 */     primitiveNames.add(Character.TYPE.getName());
/*  66 */     primitiveNames.add(Boolean.TYPE.getName());
/*  67 */     PRIMITIVE_NAMES = Collections.unmodifiableSet(primitiveNames);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Property shallowCopy(Property property)
/*     */   {
/*  76 */     Property clone = new Property();
/*  77 */     clone.setCascade(property.getCascade());
/*  78 */     clone.setInsertable(property.isInsertable());
/*  79 */     clone.setLazy(property.isLazy());
/*  80 */     clone.setName(property.getName());
/*  81 */     clone.setNodeName(property.getNodeName());
/*  82 */     clone.setNaturalIdentifier(property.isNaturalIdentifier());
/*  83 */     clone.setOptimisticLocked(property.isOptimisticLocked());
/*  84 */     clone.setOptional(property.isOptional());
/*  85 */     clone.setPersistentClass(property.getPersistentClass());
/*  86 */     clone.setPropertyAccessorName(property.getPropertyAccessorName());
/*  87 */     clone.setSelectable(property.isSelectable());
/*  88 */     clone.setUpdateable(property.isUpdateable());
/*  89 */     clone.setValue(property.getValue());
/*  90 */     return clone;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static void createSyntheticPropertyReference(Ejb3JoinColumn[] columns, PersistentClass ownerEntity, PersistentClass associatedEntity, Value value, boolean inverse, ExtendedMappings mappings)
/*     */   {
/* 101 */     if ((columns[0].isImplicit()) || (StringHelper.isNotEmpty(columns[0].getMappedBy()))) return;
/* 102 */     int fkEnum = Ejb3JoinColumn.checkReferencedColumnsType(columns, ownerEntity, mappings);
/* 103 */     PersistentClass associatedClass = columns[0].getPropertyHolder() != null ? columns[0].getPropertyHolder().getPersistentClass() : null;
/*     */     
/*     */ 
/* 106 */     if (2 == fkEnum)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 114 */       StringBuilder propertyNameBuffer = new StringBuilder("_");
/* 115 */       propertyNameBuffer.append(associatedClass.getEntityName().replace('.', '_'));
/* 116 */       propertyNameBuffer.append("_").append(columns[0].getPropertyName());
/* 117 */       String syntheticPropertyName = propertyNameBuffer.toString();
/*     */       
/* 119 */       Object columnOwner = findColumnOwner(ownerEntity, columns[0].getReferencedColumn(), mappings);
/* 120 */       List<Property> properties = findPropertiesByColumns(columnOwner, columns, mappings);
/*     */       
/* 122 */       Property synthProp = null;
/* 123 */       if (properties != null)
/*     */       {
/* 125 */         Component embeddedComp = (columnOwner instanceof PersistentClass) ? new Component((PersistentClass)columnOwner) : new Component((Join)columnOwner);
/*     */         
/*     */ 
/* 128 */         embeddedComp.setEmbedded(true);
/* 129 */         embeddedComp.setNodeName(syntheticPropertyName);
/* 130 */         embeddedComp.setComponentClassName(embeddedComp.getOwner().getClassName());
/* 131 */         for (Property property : properties) {
/* 132 */           Property clone = shallowCopy(property);
/* 133 */           clone.setInsertable(false);
/* 134 */           clone.setUpdateable(false);
/* 135 */           clone.setNaturalIdentifier(false);
/* 136 */           embeddedComp.addProperty(clone);
/*     */         }
/* 138 */         synthProp = new Property();
/* 139 */         synthProp.setName(syntheticPropertyName);
/* 140 */         synthProp.setNodeName(syntheticPropertyName);
/* 141 */         synthProp.setPersistentClass(ownerEntity);
/* 142 */         synthProp.setUpdateable(false);
/* 143 */         synthProp.setInsertable(false);
/* 144 */         synthProp.setValue(embeddedComp);
/* 145 */         synthProp.setPropertyAccessorName("embedded");
/* 146 */         ownerEntity.addProperty(synthProp);
/*     */         
/* 148 */         TableBinder.createUniqueConstraint(embeddedComp);
/*     */       }
/*     */       else
/*     */       {
/* 152 */         StringBuilder columnsList = new StringBuilder();
/* 153 */         columnsList.append("referencedColumnNames(");
/* 154 */         for (Ejb3JoinColumn column : columns) {
/* 155 */           columnsList.append(column.getReferencedColumn()).append(", ");
/*     */         }
/* 157 */         columnsList.setLength(columnsList.length() - 2);
/* 158 */         columnsList.append(") ");
/*     */         
/* 160 */         if (associatedEntity != null)
/*     */         {
/* 162 */           columnsList.append("of ").append(associatedEntity.getEntityName()).append(".").append(columns[0].getPropertyName()).append(" ");
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         }
/* 169 */         else if (columns[0].getPropertyHolder() != null) {
/* 170 */           columnsList.append("of ").append(columns[0].getPropertyHolder().getEntityName()).append(".").append(columns[0].getPropertyName()).append(" ");
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 177 */         columnsList.append("referencing ").append(ownerEntity.getEntityName()).append(" not mapped to a single property");
/*     */         
/*     */ 
/* 180 */         throw new AnnotationException(columnsList.toString());
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 186 */       if ((value instanceof ToOne)) {
/* 187 */         ((ToOne)value).setReferencedPropertyName(syntheticPropertyName);
/* 188 */         mappings.addUniquePropertyReference(ownerEntity.getEntityName(), syntheticPropertyName);
/*     */       }
/* 190 */       else if ((value instanceof Collection)) {
/* 191 */         ((Collection)value).setReferencedPropertyName(syntheticPropertyName);
/*     */         
/* 193 */         mappings.addPropertyReference(ownerEntity.getEntityName(), syntheticPropertyName);
/*     */       }
/*     */       else {
/* 196 */         throw new AssertionFailure("Do a property ref on an unexpected Value type: " + value.getClass().getName());
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 201 */       mappings.addPropertyReferencedAssociation((inverse ? "inverse__" : "") + associatedClass.getEntityName(), columns[0].getPropertyName(), syntheticPropertyName);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static List<Property> findPropertiesByColumns(Object columnOwner, Ejb3JoinColumn[] columns, ExtendedMappings mappings)
/*     */   {
/* 214 */     Map<org.hibernate.mapping.Column, Set<Property>> columnsToProperty = new HashMap();
/* 215 */     List<org.hibernate.mapping.Column> orderedColumns = new ArrayList(columns.length);
/* 216 */     Table referencedTable = null;
/* 217 */     if ((columnOwner instanceof PersistentClass)) {
/* 218 */       referencedTable = ((PersistentClass)columnOwner).getTable();
/*     */     }
/* 220 */     else if ((columnOwner instanceof Join)) {
/* 221 */       referencedTable = ((Join)columnOwner).getTable();
/*     */     }
/*     */     else {
/* 224 */       throw new AssertionFailure("columnOwner neither PersistentClass nor Join: " + columnOwner.getClass());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 231 */     for (Ejb3JoinColumn column1 : columns) {
/* 232 */       org.hibernate.mapping.Column column = new org.hibernate.mapping.Column(mappings.getPhysicalColumnName(column1.getReferencedColumn(), referencedTable));
/*     */       
/*     */ 
/* 235 */       orderedColumns.add(column);
/* 236 */       columnsToProperty.put(column, new HashSet());
/*     */     }
/* 238 */     boolean isPersistentClass = columnOwner instanceof PersistentClass;
/* 239 */     Iterator it = isPersistentClass ? ((PersistentClass)columnOwner).getPropertyIterator() : ((Join)columnOwner).getPropertyIterator();
/*     */     
/*     */ 
/* 242 */     while (it.hasNext()) {
/* 243 */       matchColumnsByProperty((Property)it.next(), columnsToProperty);
/*     */     }
/* 245 */     if (isPersistentClass) {
/* 246 */       matchColumnsByProperty(((PersistentClass)columnOwner).getIdentifierProperty(), columnsToProperty);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 252 */     List<Property> orderedProperties = new ArrayList();
/* 253 */     for (org.hibernate.mapping.Column column : orderedColumns) {
/* 254 */       boolean found = false;
/* 255 */       for (Property property : (Set)columnsToProperty.get(column)) {
/* 256 */         if (property.getColumnSpan() == 1) {
/* 257 */           orderedProperties.add(property);
/* 258 */           found = true;
/* 259 */           break;
/*     */         }
/*     */       }
/* 262 */       if (!found) return null;
/*     */     }
/* 264 */     return orderedProperties;
/*     */   }
/*     */   
/*     */   private static void matchColumnsByProperty(Property property, Map<org.hibernate.mapping.Column, Set<Property>> columnsToProperty) {
/* 268 */     if (property == null) return;
/* 269 */     if (("noop".equals(property.getPropertyAccessorName())) || ("embedded".equals(property.getPropertyAccessorName())))
/*     */     {
/* 271 */       return;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 281 */     Iterator columnIt = property.getColumnIterator();
/* 282 */     while (columnIt.hasNext()) {
/* 283 */       Object column = columnIt.next();
/*     */       
/* 285 */       if (columnsToProperty.containsKey(column)) {
/* 286 */         ((Set)columnsToProperty.get(column)).add(property);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Property findPropertyByName(PersistentClass associatedClass, String propertyName)
/*     */   {
/* 297 */     Property property = null;
/* 298 */     Property idProperty = associatedClass.getIdentifierProperty();
/* 299 */     String idName = idProperty != null ? idProperty.getName() : null;
/*     */     try {
/* 301 */       if ((propertyName == null) || (propertyName.length() == 0) || (propertyName.equals(idName)))
/*     */       {
/*     */ 
/*     */ 
/* 305 */         property = idProperty;
/*     */       }
/*     */       else {
/* 308 */         if (propertyName.indexOf(idName + ".") == 0) {
/* 309 */           property = idProperty;
/* 310 */           propertyName = propertyName.substring(idName.length() + 1);
/*     */         }
/* 312 */         StringTokenizer st = new StringTokenizer(propertyName, ".", false);
/* 313 */         while (st.hasMoreElements()) {
/* 314 */           String element = (String)st.nextElement();
/* 315 */           if (property == null) {
/* 316 */             property = associatedClass.getProperty(element);
/*     */           }
/*     */           else {
/* 319 */             if (!property.isComposite()) return null;
/* 320 */             property = ((Component)property.getValue()).getProperty(element);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     catch (MappingException e)
/*     */     {
/*     */       try {
/* 328 */         if (associatedClass.getIdentifierMapper() == null) return null;
/* 329 */         StringTokenizer st = new StringTokenizer(propertyName, ".", false);
/* 330 */         while (st.hasMoreElements()) {
/* 331 */           String element = (String)st.nextElement();
/* 332 */           if (property == null) {
/* 333 */             property = associatedClass.getIdentifierMapper().getProperty(element);
/*     */           }
/*     */           else {
/* 336 */             if (!property.isComposite()) return null;
/* 337 */             property = ((Component)property.getValue()).getProperty(element);
/*     */           }
/*     */         }
/*     */       }
/*     */       catch (MappingException ee) {
/* 342 */         return null;
/*     */       }
/*     */     }
/* 345 */     return property;
/*     */   }
/*     */   
/*     */   public static String getRelativePath(PropertyHolder propertyHolder, String propertyName) {
/* 349 */     if (propertyHolder == null) return propertyName;
/* 350 */     String path = propertyHolder.getPath();
/* 351 */     String entityName = propertyHolder.getPersistentClass().getEntityName();
/* 352 */     if (path.length() == entityName.length()) {
/* 353 */       return propertyName;
/*     */     }
/*     */     
/* 356 */     return StringHelper.qualify(path.substring(entityName.length() + 1), propertyName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Object findColumnOwner(PersistentClass persistentClass, String columnName, ExtendedMappings mappings)
/*     */   {
/* 367 */     if (StringHelper.isEmpty(columnName)) {
/* 368 */       return persistentClass;
/*     */     }
/* 370 */     PersistentClass current = persistentClass;
/* 371 */     Object result = null;
/* 372 */     boolean found = false;
/*     */     do {
/* 374 */       result = current;
/* 375 */       Table currentTable = current.getTable();
/*     */       try {
/* 377 */         mappings.getPhysicalColumnName(columnName, currentTable);
/* 378 */         found = true;
/*     */       }
/*     */       catch (MappingException me) {}
/*     */       
/*     */ 
/* 383 */       Iterator joins = current.getJoinIterator();
/* 384 */       while ((!found) && (joins.hasNext())) {
/* 385 */         result = joins.next();
/* 386 */         currentTable = ((Join)result).getTable();
/*     */         try {
/* 388 */           mappings.getPhysicalColumnName(columnName, currentTable);
/* 389 */           found = true;
/*     */         }
/*     */         catch (MappingException me) {}
/*     */       }
/*     */       
/*     */ 
/* 395 */       current = current.getSuperclass();
/*     */     }
/* 397 */     while ((!found) && (current != null));
/* 398 */     return found ? result : null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static void makeIdGenerator(SimpleValue id, String generatorType, String generatorName, ExtendedMappings mappings, Map<String, IdGenerator> localGenerators)
/*     */   {
/* 408 */     Table table = id.getTable();
/* 409 */     table.setIdentifierValue(id);
/*     */     
/* 411 */     id.setIdentifierGeneratorStrategy(generatorType);
/* 412 */     Properties params = new Properties();
/*     */     
/* 414 */     params.setProperty("target_table", table.getName());
/*     */     
/*     */ 
/*     */ 
/* 418 */     if (id.getColumnSpan() == 1) {
/* 419 */       params.setProperty("target_column", ((org.hibernate.mapping.Column)id.getColumnIterator().next()).getName());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 424 */     if (!isDefault(generatorName))
/*     */     {
/* 426 */       IdGenerator gen = mappings.getGenerator(generatorName, localGenerators);
/* 427 */       if (gen == null) {
/* 428 */         throw new AnnotationException("Unknown Id.generator: " + generatorName);
/*     */       }
/*     */       
/* 431 */       String identifierGeneratorStrategy = gen.getIdentifierGeneratorStrategy();
/*     */       
/* 433 */       boolean avoidOverriding = (identifierGeneratorStrategy.equals("identity")) || (identifierGeneratorStrategy.equals("seqhilo")) || (identifierGeneratorStrategy.equals(MultipleHiLoPerTableGenerator.class.getName()));
/*     */       
/*     */ 
/*     */ 
/* 437 */       if ((generatorType == null) || (!avoidOverriding)) {
/* 438 */         id.setIdentifierGeneratorStrategy(identifierGeneratorStrategy);
/*     */       }
/*     */       
/* 441 */       Iterator genParams = gen.getParams().entrySet().iterator();
/* 442 */       while (genParams.hasNext()) {
/* 443 */         Map.Entry elt = (Map.Entry)genParams.next();
/* 444 */         params.setProperty((String)elt.getKey(), (String)elt.getValue());
/*     */       }
/*     */     }
/* 447 */     if ("assigned".equals(generatorType)) id.setNullValue("undefined");
/* 448 */     id.setIdentifierGeneratorProperties(params);
/*     */   }
/*     */   
/*     */   public static boolean isDefault(String annotationString) {
/* 452 */     return (annotationString != null) && (annotationString.length() == 0);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Any buildAnyValue(String anyMetaDefName, Ejb3JoinColumn[] columns, javax.persistence.Column metaColumn, PropertyData inferredData, boolean cascadeOnDelete, Nullability nullability, PropertyHolder propertyHolder, EntityBinder entityBinder, boolean optional, ExtendedMappings mappings)
/*     */   {
/* 460 */     Any value = new Any(columns[0].getTable());
/* 461 */     AnyMetaDef metaAnnDef = (AnyMetaDef)inferredData.getProperty().getAnnotation(AnyMetaDef.class);
/*     */     
/* 463 */     if (metaAnnDef != null)
/*     */     {
/* 465 */       bindAnyMetaDefs(inferredData.getProperty(), mappings);
/*     */     }
/*     */     else {
/* 468 */       metaAnnDef = mappings.getAnyMetaDef(anyMetaDefName);
/*     */     }
/* 470 */     if (metaAnnDef != null) {
/* 471 */       value.setIdentifierType(metaAnnDef.idType());
/* 472 */       value.setMetaType(metaAnnDef.metaType());
/*     */       
/* 474 */       HashMap values = new HashMap();
/* 475 */       Type metaType = TypeFactory.heuristicType(value.getMetaType());
/* 476 */       for (MetaValue metaValue : metaAnnDef.metaValues()) {
/*     */         try {
/* 478 */           Object discrim = ((DiscriminatorType)metaType).stringToObject(metaValue.value());
/*     */           
/* 480 */           String entityName = metaValue.targetEntity().getName();
/* 481 */           values.put(discrim, entityName);
/*     */         }
/*     */         catch (ClassCastException cce) {
/* 484 */           throw new MappingException("metaType was not a DiscriminatorType: " + metaType.getName());
/*     */         }
/*     */         catch (Exception e)
/*     */         {
/* 488 */           throw new MappingException("could not interpret metaValue", e);
/*     */         }
/*     */       }
/* 491 */       if (!values.isEmpty()) value.setMetaValues(values);
/*     */     }
/*     */     else {
/* 494 */       throw new AnnotationException("Unable to find @AnyMetaDef for an @(ManyTo)Any mapping: " + StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*     */     }
/*     */     
/*     */ 
/* 498 */     value.setCascadeDeleteEnabled(cascadeOnDelete);
/* 499 */     if (!optional) {
/* 500 */       for (Ejb3JoinColumn column : columns) {
/* 501 */         column.setNullable(false);
/*     */       }
/*     */     }
/*     */     
/* 505 */     Ejb3Column[] metaColumns = Ejb3Column.buildColumnFromAnnotation(new javax.persistence.Column[] { metaColumn }, null, nullability, propertyHolder, inferredData, entityBinder.getSecondaryTables(), mappings);
/*     */     
/*     */ 
/* 508 */     for (Ejb3Column column : metaColumns) {
/* 509 */       column.setTable(value.getTable());
/*     */     }
/*     */     
/* 512 */     for (Ejb3Column column : metaColumns) {
/* 513 */       column.linkWithValue(value);
/*     */     }
/*     */     
/*     */ 
/* 517 */     String propertyName = inferredData.getPropertyName();
/* 518 */     Ejb3Column.checkPropertyConsistency(columns, propertyHolder.getEntityName() + propertyName);
/* 519 */     for (Ejb3JoinColumn column : columns) {
/* 520 */       column.linkWithValue(value);
/*     */     }
/* 522 */     return value;
/*     */   }
/*     */   
/*     */   public static void bindAnyMetaDefs(XAnnotatedElement annotatedElement, ExtendedMappings mappings) {
/* 526 */     AnyMetaDef defAnn = (AnyMetaDef)annotatedElement.getAnnotation(AnyMetaDef.class);
/* 527 */     AnyMetaDefs defsAnn = (AnyMetaDefs)annotatedElement.getAnnotation(AnyMetaDefs.class);
/* 528 */     boolean mustHaveName = (XClass.class.isAssignableFrom(annotatedElement.getClass())) || (XPackage.class.isAssignableFrom(annotatedElement.getClass()));
/*     */     
/* 530 */     if (defAnn != null) {
/* 531 */       checkAnyMetaDefValidity(mustHaveName, defAnn, annotatedElement);
/* 532 */       bindAnyMetaDef(defAnn, mappings);
/*     */     }
/* 534 */     if (defsAnn != null) {
/* 535 */       for (AnyMetaDef def : defsAnn.value()) {
/* 536 */         checkAnyMetaDefValidity(mustHaveName, def, annotatedElement);
/* 537 */         bindAnyMetaDef(def, mappings);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private static void checkAnyMetaDefValidity(boolean mustHaveName, AnyMetaDef defAnn, XAnnotatedElement annotatedElement) {
/* 543 */     if ((mustHaveName) && (isDefault(defAnn.name()))) {
/* 544 */       String name = XClass.class.isAssignableFrom(annotatedElement.getClass()) ? ((XClass)annotatedElement).getName() : ((XPackage)annotatedElement).getName();
/*     */       
/*     */ 
/* 547 */       throw new AnnotationException("@AnyMetaDef.name cannot be null on an entity or a package: " + name);
/*     */     }
/*     */   }
/*     */   
/*     */   private static void bindAnyMetaDef(AnyMetaDef defAnn, ExtendedMappings mappings) {
/* 552 */     if (isDefault(defAnn.name())) return;
/* 553 */     if (log.isInfoEnabled()) log.info("Binding Any Meta definition: " + defAnn.name());
/* 554 */     mappings.addAnyMetaDef(defAnn);
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\BinderHelper.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */