/*      */ package org.hibernate.ejb;
/*      */ 
/*      */ import java.io.BufferedInputStream;
/*      */ import java.io.ByteArrayOutputStream;
/*      */ import java.io.File;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.ObjectOutput;
/*      */ import java.io.ObjectOutputStream;
/*      */ import java.io.Serializable;
/*      */ import java.net.URL;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Collections;
/*      */ import java.util.Enumeration;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Properties;
/*      */ import java.util.Set;
/*      */ import java.util.StringTokenizer;
/*      */ import javax.naming.BinaryRefAddr;
/*      */ import javax.naming.NamingException;
/*      */ import javax.naming.Reference;
/*      */ import javax.naming.Referenceable;
/*      */ import javax.persistence.Embeddable;
/*      */ import javax.persistence.Entity;
/*      */ import javax.persistence.EntityManagerFactory;
/*      */ import javax.persistence.EntityNotFoundException;
/*      */ import javax.persistence.MappedSuperclass;
/*      */ import javax.persistence.PersistenceException;
/*      */ import javax.persistence.spi.PersistenceUnitInfo;
/*      */ import javax.persistence.spi.PersistenceUnitTransactionType;
/*      */ import javax.sql.DataSource;
/*      */ import org.apache.commons.logging.Log;
/*      */ import org.apache.commons.logging.LogFactory;
/*      */ import org.dom4j.Document;
/*      */ import org.dom4j.DocumentException;
/*      */ import org.dom4j.Element;
/*      */ import org.dom4j.io.SAXReader;
/*      */ import org.hibernate.HibernateException;
/*      */ import org.hibernate.Interceptor;
/*      */ import org.hibernate.MappingException;
/*      */ import org.hibernate.MappingNotFoundException;
/*      */ import org.hibernate.SessionFactory;
/*      */ import org.hibernate.cfg.AnnotationConfiguration;
/*      */ import org.hibernate.cfg.Configuration;
/*      */ import org.hibernate.cfg.NamingStrategy;
/*      */ import org.hibernate.cfg.Settings;
/*      */ import org.hibernate.cfg.SettingsFactory;
/*      */ import org.hibernate.cfg.annotations.reflection.XMLContext;
/*      */ import org.hibernate.ejb.connection.InjectedDataSourceConnectionProvider;
/*      */ import org.hibernate.ejb.instrument.InterceptFieldClassFileTransformer;
/*      */ import org.hibernate.ejb.packaging.ClassFilter;
/*      */ import org.hibernate.ejb.packaging.Entry;
/*      */ import org.hibernate.ejb.packaging.FileFilter;
/*      */ import org.hibernate.ejb.packaging.Filter;
/*      */ import org.hibernate.ejb.packaging.JarVisitor;
/*      */ import org.hibernate.ejb.packaging.JarVisitorFactory;
/*      */ import org.hibernate.ejb.packaging.NamedInputStream;
/*      */ import org.hibernate.ejb.packaging.PackageFilter;
/*      */ import org.hibernate.ejb.packaging.PersistenceMetadata;
/*      */ import org.hibernate.ejb.packaging.PersistenceXmlLoader;
/*      */ import org.hibernate.ejb.transaction.JoinableCMTTransactionFactory;
/*      */ import org.hibernate.ejb.util.ConfigurationHelper;
/*      */ import org.hibernate.ejb.util.LogHelper;
/*      */ import org.hibernate.ejb.util.NamingHelper;
/*      */ import org.hibernate.engine.FilterDefinition;
/*      */ import org.hibernate.event.EventListeners;
/*      */ import org.hibernate.mapping.AuxiliaryDatabaseObject;
/*      */ import org.hibernate.mapping.PersistentClass;
/*      */ import org.hibernate.proxy.EntityNotFoundDelegate;
/*      */ import org.hibernate.secure.JACCConfiguration;
/*      */ import org.hibernate.transaction.JDBCTransactionFactory;
/*      */ import org.hibernate.util.CollectionHelper;
/*      */ import org.hibernate.util.ReflectHelper;
/*      */ import org.hibernate.util.StringHelper;
/*      */ import org.hibernate.util.XMLHelper;
/*      */ import org.xml.sax.EntityResolver;
/*      */ import org.xml.sax.SAXException;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class Ejb3Configuration
/*      */   implements Serializable, Referenceable
/*      */ {
/*  103 */   private static final String IMPLEMENTATION_NAME = HibernatePersistence.class.getName();
/*      */   private static final String META_INF_ORM_XML = "META-INF/orm.xml";
/*  105 */   private static Log log = LogFactory.getLog(Ejb3Configuration.class);
/*  106 */   private static EntityNotFoundDelegate ejb3EntityNotFoundDelegate = new Ejb3EntityNotFoundDelegate(null);
/*  107 */   private static Configuration DEFAULT_CONFIGURATION = new AnnotationConfiguration();
/*      */   private String persistenceUnitName;
/*      */   private String cfgXmlResource;
/*      */   private AnnotationConfiguration cfg;
/*      */   private SettingsFactory settingsFactory;
/*      */   
/*  113 */   private static class Ejb3EntityNotFoundDelegate implements EntityNotFoundDelegate, Serializable { public void handleEntityNotFound(String entityName, Serializable id) { throw new EntityNotFoundException("Unable to find " + entityName + " with id " + id); }
/*      */   }
/*      */   
/*      */   static
/*      */   {
/*  118 */     Version.touch();
/*      */   }
/*      */   
/*      */ 
/*      */   private transient EventListenerConfigurator listenerConfigurator;
/*      */   
/*      */   private PersistenceUnitTransactionType transactionType;
/*      */   
/*      */   private boolean discardOnClose;
/*      */   
/*      */   private transient ClassLoader overridenClassLoader;
/*  129 */   private boolean isConfigurationProcessed = false;
/*      */   
/*      */   public Ejb3Configuration()
/*      */   {
/*  133 */     this.settingsFactory = new InjectionSettingsFactory();
/*  134 */     this.cfg = new AnnotationConfiguration(this.settingsFactory);
/*  135 */     this.cfg.setEntityNotFoundDelegate(ejb3EntityNotFoundDelegate);
/*  136 */     this.listenerConfigurator = new EventListenerConfigurator(this);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setDataSource(DataSource ds)
/*      */   {
/*  145 */     if (ds != null) {
/*  146 */       Map cpInjection = new HashMap();
/*  147 */       cpInjection.put("dataSource", ds);
/*  148 */       ((InjectionSettingsFactory)this.settingsFactory).setConnectionProviderInjectionData(cpInjection);
/*  149 */       setProperty("hibernate.connection.provider_class", InjectedDataSourceConnectionProvider.class.getName());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private Ejb3Configuration configure(PersistenceMetadata metadata, Map overrides)
/*      */   {
/*  158 */     log.debug("Creating Factory: " + metadata.getName());
/*      */     
/*  160 */     Map workingVars = new HashMap();
/*  161 */     workingVars.put("hibernate.ejb.persistenceUnitName", metadata.getName());
/*  162 */     this.persistenceUnitName = metadata.getName();
/*      */     
/*  164 */     if (StringHelper.isNotEmpty(metadata.getJtaDatasource())) {
/*  165 */       setProperty("hibernate.connection.datasource", metadata.getJtaDatasource());
/*      */     }
/*  167 */     else if (StringHelper.isNotEmpty(metadata.getNonJtaDatasource())) {
/*  168 */       setProperty("hibernate.connection.datasource", metadata.getNonJtaDatasource());
/*      */     }
/*  170 */     defineTransactionType(metadata.getTransactionType(), workingVars);
/*  171 */     if (metadata.getClasses().size() > 0) {
/*  172 */       workingVars.put("hibernate.ejb.classes", metadata.getClasses());
/*      */     }
/*  174 */     if (metadata.getPackages().size() > 0) {
/*  175 */       workingVars.put("hibernate.ejb.packages", metadata.getPackages());
/*      */     }
/*  177 */     if (metadata.getMappingFiles().size() > 0) {
/*  178 */       workingVars.put("hibernate.ejb.xml_files", metadata.getMappingFiles());
/*      */     }
/*  180 */     if (metadata.getHbmfiles().size() > 0) {
/*  181 */       workingVars.put("hibernate.hbmxml.files", metadata.getHbmfiles());
/*      */     }
/*  183 */     Properties props = new Properties();
/*  184 */     props.putAll(metadata.getProps());
/*  185 */     if (overrides != null) {
/*  186 */       for (Map.Entry entry : overrides.entrySet()) {
/*  187 */         Object value = entry.getValue();
/*  188 */         props.put(entry.getKey(), value == null ? "" : value);
/*      */       }
/*      */     }
/*  191 */     configure(props, workingVars);
/*  192 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Ejb3Configuration configure(String persistenceUnitName, Map integration)
/*      */   {
/*      */     try
/*      */     {
/*  209 */       log.debug("Look up for persistence unit: " + persistenceUnitName);
/*  210 */       integration = integration == null ? CollectionHelper.EMPTY_MAP : Collections.unmodifiableMap(integration);
/*      */       
/*      */ 
/*  213 */       Enumeration<URL> xmls = Thread.currentThread().getContextClassLoader().getResources("META-INF/persistence.xml");
/*      */       
/*      */ 
/*  216 */       if (!xmls.hasMoreElements())
/*  217 */         log.info("Could not find any META-INF/persistence.xml file in the classpath");
/*      */       URL url;
/*  219 */       while (xmls.hasMoreElements()) {
/*  220 */         url = (URL)xmls.nextElement();
/*  221 */         log.trace("Analysing persistence.xml: " + url);
/*  222 */         List<PersistenceMetadata> metadataFiles = PersistenceXmlLoader.deploy(url, integration, this.cfg.getEntityResolver(), PersistenceUnitTransactionType.RESOURCE_LOCAL);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*  227 */         for (PersistenceMetadata metadata : metadataFiles) {
/*  228 */           log.trace(metadata.toString());
/*      */           
/*  230 */           if ((metadata.getProvider() == null) || (IMPLEMENTATION_NAME.equalsIgnoreCase(metadata.getProvider())))
/*      */           {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  236 */             JarVisitor visitor = null;
/*      */             
/*  238 */             if (metadata.getName() == null) {
/*  239 */               visitor = getMainJarVisitor(url, metadata, integration);
/*  240 */               metadata.setName(visitor.getUnqualifiedJarName());
/*      */             }
/*  242 */             if ((persistenceUnitName == null) && (xmls.hasMoreElements())) {
/*  243 */               throw new PersistenceException("No name provided and several persistence units found");
/*      */             }
/*  245 */             if ((persistenceUnitName == null) || (metadata.getName().equals(persistenceUnitName))) {
/*  246 */               if (visitor == null) visitor = getMainJarVisitor(url, metadata, integration);
/*  247 */               addMetadataFromVisitor(visitor, metadata);
/*  248 */               Filter[] otherXmlFilter = getFilters(metadata, integration, false);
/*  249 */               for (String jarFile : metadata.getJarFiles()) {
/*  250 */                 visitor = JarVisitorFactory.getVisitor(jarFile, otherXmlFilter);
/*  251 */                 addMetadataFromVisitor(visitor, metadata);
/*      */               }
/*  253 */               return configure(metadata, integration);
/*      */             }
/*      */           }
/*      */         }
/*      */       }
/*  258 */       return null;
/*      */     }
/*      */     catch (Exception e) {
/*  261 */       if ((e instanceof PersistenceException)) {
/*  262 */         throw ((PersistenceException)e);
/*      */       }
/*      */       
/*  265 */       throw new PersistenceException(getExceptionHeader() + "Unable to configure EntityManagerFactory", e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private JarVisitor getMainJarVisitor(URL url, PersistenceMetadata metadata, Map integration)
/*      */   {
/*  272 */     URL jarURL = JarVisitorFactory.getJarURLFromURLEntry(url, "/META-INF/persistence.xml");
/*  273 */     Filter[] persistenceXmlFilter = getFilters(metadata, integration, metadata.getExcludeUnlistedClasses());
/*  274 */     return JarVisitorFactory.getVisitor(jarURL, persistenceXmlFilter);
/*      */   }
/*      */   
/*      */   private static void addMetadataFromVisitor(JarVisitor visitor, PersistenceMetadata metadata) throws IOException {
/*  278 */     List<String> classes = metadata.getClasses();
/*  279 */     List<String> packages = metadata.getPackages();
/*  280 */     List<NamedInputStream> hbmFiles = metadata.getHbmfiles();
/*  281 */     List<String> mappingFiles = metadata.getMappingFiles();
/*  282 */     addScannedEntries(visitor, classes, packages, hbmFiles, mappingFiles);
/*      */   }
/*      */   
/*      */   private static void addScannedEntries(JarVisitor visitor, List<String> classes, List<String> packages, List<NamedInputStream> hbmFiles, List<String> mappingFiles) throws IOException {
/*  286 */     Filter[] filters = visitor.getFilters();
/*  287 */     Set[] entries = visitor.getMatchingEntries();
/*  288 */     int size = filters.length;
/*  289 */     for (int index = 0; index < size; index++) {
/*  290 */       for (Object o : entries[index]) {
/*  291 */         Entry entry = (Entry)o;
/*  292 */         if ((filters[index] instanceof ClassFilter)) {
/*  293 */           classes.add(entry.getName());
/*      */         }
/*  295 */         else if ((filters[index] instanceof PackageFilter)) {
/*  296 */           packages.add(entry.getName());
/*      */         }
/*  298 */         else if ((filters[index] instanceof FileFilter)) {
/*  299 */           hbmFiles.add(new NamedInputStream(entry.getName(), entry.getInputStream()));
/*  300 */           if (mappingFiles != null) { mappingFiles.remove(entry.getName());
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public Ejb3Configuration configure(PersistenceUnitInfo info, Map integration)
/*      */   {
/*  311 */     if (log.isDebugEnabled()) {
/*  312 */       log.debug("Processing " + LogHelper.logPersistenceUnitInfo(info));
/*      */     }
/*      */     else {
/*  315 */       log.info("Processing PersistenceUnitInfo [\n\tname: " + info.getPersistenceUnitName() + "\n\t...]");
/*      */     }
/*      */     
/*  318 */     integration = integration != null ? Collections.unmodifiableMap(integration) : CollectionHelper.EMPTY_MAP;
/*  319 */     String provider = (String)integration.get("javax.persistence.provider");
/*  320 */     if (provider == null) provider = info.getPersistenceProviderClassName();
/*  321 */     if ((provider != null) && (!provider.trim().startsWith(IMPLEMENTATION_NAME))) {
/*  322 */       log.info("Required a different provider: " + provider);
/*  323 */       return null;
/*      */     }
/*  325 */     if (info.getClassLoader() == null) {
/*  326 */       throw new IllegalStateException(info.getPersistenceUnitName() + "] " + "PersistenceUnitInfo.getClassLoader() id null");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  331 */     Thread thread = Thread.currentThread();
/*  332 */     ClassLoader contextClassLoader = thread.getContextClassLoader();
/*  333 */     boolean sameClassLoader = info.getClassLoader().equals(contextClassLoader);
/*  334 */     if (!sameClassLoader) {
/*  335 */       this.overridenClassLoader = info.getClassLoader();
/*  336 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     else {
/*  339 */       this.overridenClassLoader = null;
/*      */     }
/*      */     try
/*      */     {
/*  343 */       Map workingVars = new HashMap();
/*  344 */       workingVars.put("hibernate.ejb.persistenceUnitName", info.getPersistenceUnitName());
/*  345 */       this.persistenceUnitName = info.getPersistenceUnitName();
/*  346 */       List<String> entities = new ArrayList(50);
/*  347 */       if (info.getManagedClassNames() != null) entities.addAll(info.getManagedClassNames());
/*  348 */       List<NamedInputStream> hbmFiles = new ArrayList();
/*  349 */       List<String> packages = new ArrayList();
/*  350 */       List<String> xmlFiles = new ArrayList(50);
/*  351 */       if (info.getMappingFileNames() != null) { xmlFiles.addAll(info.getMappingFileNames());
/*      */       }
/*  353 */       boolean searchForORMFiles = !xmlFiles.contains("META-INF/orm.xml");
/*      */       
/*  355 */       boolean[] detectArtifactForOtherJars = getDetectedArtifacts(info.getProperties(), null, false);
/*  356 */       boolean[] detectArtifactForMainJar = getDetectedArtifacts(info.getProperties(), null, info.excludeUnlistedClasses());
/*  357 */       for (URL jar : info.getJarFileUrls()) {
/*  358 */         scanForClasses(jar, packages, entities, hbmFiles, detectArtifactForOtherJars, searchForORMFiles);
/*      */       }
/*  360 */       scanForClasses(info.getPersistenceUnitRootUrl(), packages, entities, hbmFiles, detectArtifactForMainJar, searchForORMFiles);
/*      */       
/*  362 */       Properties properties = info.getProperties() != null ? info.getProperties() : new Properties();
/*      */       
/*      */ 
/*  365 */       ConfigurationHelper.overrideProperties(properties, integration);
/*      */       
/*      */ 
/*      */ 
/*  369 */       addXMLEntities(xmlFiles, info, entities);
/*      */       
/*      */ 
/*  372 */       if ("true".equalsIgnoreCase(properties.getProperty("hibernate.ejb.use_class_enhancer"))) {
/*  373 */         info.addTransformer(new InterceptFieldClassFileTransformer(entities));
/*      */       }
/*      */       
/*  376 */       workingVars.put("hibernate.ejb.classes", entities);
/*  377 */       workingVars.put("hibernate.ejb.packages", packages);
/*  378 */       workingVars.put("hibernate.ejb.xml_files", xmlFiles);
/*  379 */       if (hbmFiles.size() > 0) { workingVars.put("hibernate.hbmxml.files", hbmFiles);
/*      */       }
/*      */       
/*  382 */       Boolean isJTA = null;
/*  383 */       boolean overridenDatasource = false;
/*  384 */       if (integration.containsKey("javax.persistence.jtaDataSource")) {
/*  385 */         String dataSource = (String)integration.get("javax.persistence.jtaDataSource");
/*  386 */         overridenDatasource = true;
/*  387 */         properties.setProperty("hibernate.connection.datasource", dataSource);
/*  388 */         isJTA = Boolean.TRUE;
/*      */       }
/*  390 */       if (integration.containsKey("javax.persistence.nonJtaDataSource")) {
/*  391 */         String dataSource = (String)integration.get("javax.persistence.nonJtaDataSource");
/*  392 */         overridenDatasource = true;
/*  393 */         properties.setProperty("hibernate.connection.datasource", dataSource);
/*  394 */         if (isJTA == null) { isJTA = Boolean.FALSE;
/*      */         }
/*      */       }
/*  397 */       if ((!overridenDatasource) && ((info.getJtaDataSource() != null) || (info.getNonJtaDataSource() != null))) {
/*  398 */         isJTA = info.getJtaDataSource() != null ? Boolean.TRUE : Boolean.FALSE;
/*  399 */         setDataSource(isJTA.booleanValue() ? info.getJtaDataSource() : info.getNonJtaDataSource());
/*      */         
/*      */ 
/*  402 */         setProperty("hibernate.connection.provider_class", InjectedDataSourceConnectionProvider.class.getName());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  412 */       PersistenceUnitTransactionType transactionType = info.getTransactionType();
/*  413 */       if (transactionType == null) {
/*  414 */         if (isJTA == Boolean.TRUE) {
/*  415 */           transactionType = PersistenceUnitTransactionType.JTA;
/*      */         }
/*  417 */         else if (isJTA == Boolean.FALSE) {
/*  418 */           transactionType = PersistenceUnitTransactionType.RESOURCE_LOCAL;
/*      */         }
/*      */         else {
/*  421 */           transactionType = PersistenceUnitTransactionType.JTA;
/*      */         }
/*      */       }
/*  424 */       defineTransactionType(transactionType, workingVars);
/*  425 */       configure(properties, workingVars);
/*      */     }
/*      */     finally
/*      */     {
/*  429 */       if (!sameClassLoader) {
/*  430 */         thread.setContextClassLoader(contextClassLoader);
/*      */       }
/*      */     }
/*  433 */     return this;
/*      */   }
/*      */   
/*      */   private void addXMLEntities(List<String> xmlFiles, PersistenceUnitInfo info, List<String> entities)
/*      */   {
/*  438 */     ClassLoader newTempClassLoader = info.getNewTempClassLoader();
/*  439 */     if (newTempClassLoader == null) {
/*  440 */       log.warn("Persistence provider caller does not implement the EJB3 spec correctly. PersistenceUnitInfo.getNewTempClassLoader() is null.");
/*  441 */       return;
/*      */     }
/*  443 */     XMLHelper xmlHelper = new XMLHelper();
/*  444 */     List errors = new ArrayList();
/*  445 */     SAXReader saxReader = xmlHelper.createSAXReader("XML InputStream", errors, this.cfg.getEntityResolver());
/*      */     try {
/*  447 */       saxReader.setFeature("http://apache.org/xml/features/validation/schema", true);
/*      */       
/*      */ 
/*  450 */       saxReader.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", "http://java.sun.com/xml/ns/persistence/orm orm_1_0.xsd");
/*      */     }
/*      */     catch (SAXException e)
/*      */     {
/*  454 */       saxReader.setValidation(false);
/*      */     }
/*      */     
/*  457 */     Iterator i$ = xmlFiles.iterator(); for (;;) { if (i$.hasNext()) { String xmlFile = (String)i$.next();
/*      */         
/*  459 */         InputStream resourceAsStream = newTempClassLoader.getResourceAsStream(xmlFile);
/*  460 */         if (resourceAsStream != null) {
/*  461 */           BufferedInputStream is = new BufferedInputStream(resourceAsStream);
/*      */           try {
/*  463 */             errors.clear();
/*  464 */             Document doc = saxReader.read(is);
/*  465 */             if (errors.size() != 0) {
/*  466 */               throw new MappingException("invalid mapping: " + xmlFile, (Throwable)errors.get(0));
/*      */             }
/*  468 */             Element rootElement = doc.getRootElement();
/*  469 */             String defaultPackage; if ((rootElement != null) && ("entity-mappings".equals(rootElement.getName()))) {
/*  470 */               Element element = rootElement.element("package");
/*  471 */               defaultPackage = element != null ? element.getTextTrim() : null;
/*  472 */               List<Element> elements = rootElement.elements("entity");
/*  473 */               for (Element subelement : elements) {
/*  474 */                 String classname = XMLContext.buildSafeClassName(subelement.attributeValue("class"), defaultPackage);
/*  475 */                 if (!entities.contains(classname)) {
/*  476 */                   entities.add(classname);
/*      */                 }
/*      */               }
/*  479 */               elements = rootElement.elements("mapped-superclass");
/*  480 */               for (Element subelement : elements) {
/*  481 */                 String classname = XMLContext.buildSafeClassName(subelement.attributeValue("class"), defaultPackage);
/*  482 */                 if (!entities.contains(classname)) {
/*  483 */                   entities.add(classname);
/*      */                 }
/*      */               }
/*  486 */               elements = rootElement.elements("embeddable");
/*  487 */               for (Element subelement : elements) {
/*  488 */                 String classname = XMLContext.buildSafeClassName(subelement.attributeValue("class"), defaultPackage);
/*  489 */                 if (!entities.contains(classname)) {
/*  490 */                   entities.add(classname);
/*      */                 }
/*      */               }
/*      */             }
/*  494 */             else if ((rootElement == null) || (!"hibernate-mappings".equals(rootElement.getName()))) {}
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             try
/*      */             {
/*  503 */               is.close();
/*      */             }
/*      */             catch (IOException ioe) {
/*  506 */               log.warn("Could not close input stream", ioe);
/*      */             }
/*      */           }
/*      */           catch (DocumentException e)
/*      */           {
/*  499 */             throw new MappingException("Could not parse mapping document in input stream", e);
/*      */           }
/*      */           finally {
/*      */             try {
/*  503 */               is.close();
/*      */             }
/*      */             catch (IOException ioe) {
/*  506 */               log.warn("Could not close input stream", ioe);
/*      */             }
/*      */           }
/*      */         }
/*      */       }
/*      */     } }
/*      */   
/*  513 */   private void defineTransactionType(Object overridenTxType, Map workingVars) { if (overridenTxType != null)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  519 */       if ((overridenTxType instanceof String)) {
/*  520 */         this.transactionType = PersistenceXmlLoader.getTransactionType((String)overridenTxType);
/*      */       }
/*  522 */       else if ((overridenTxType instanceof PersistenceUnitTransactionType)) {
/*  523 */         this.transactionType = ((PersistenceUnitTransactionType)overridenTxType);
/*      */       }
/*      */       else {
/*  526 */         throw new PersistenceException(getExceptionHeader() + "javax.persistence.transactionType" + " of the wrong class type" + ": " + overridenTxType.getClass());
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public Ejb3Configuration setProperty(String key, String value)
/*      */   {
/*  535 */     this.cfg.setProperty(key, value);
/*  536 */     return this;
/*      */   }
/*      */   
/*      */   private boolean[] getDetectedArtifacts(Properties properties, Map overridenProperties, boolean excludeIfNotOverriden) {
/*  540 */     boolean[] result = new boolean[2];
/*  541 */     result[0] = false;
/*  542 */     result[1] = false;
/*  543 */     String detect = overridenProperties != null ? (String)overridenProperties.get("hibernate.archive.autodetection") : null;
/*      */     
/*      */ 
/*  546 */     detect = detect == null ? properties.getProperty("hibernate.archive.autodetection") : detect;
/*      */     
/*      */ 
/*  549 */     if ((detect == null) && (excludeIfNotOverriden))
/*      */     {
/*  551 */       return result;
/*      */     }
/*  553 */     if (detect == null) {
/*  554 */       detect = "class,hbm";
/*      */     }
/*  556 */     StringTokenizer st = new StringTokenizer(detect, ", ", false);
/*  557 */     while (st.hasMoreElements()) {
/*  558 */       String element = (String)st.nextElement();
/*  559 */       if ("class".equalsIgnoreCase(element)) result[0] = true;
/*  560 */       if ("hbm".equalsIgnoreCase(element)) result[1] = true;
/*      */     }
/*  562 */     log.debug("Detect class: " + result[0] + "; detect hbm: " + result[1]);
/*  563 */     return result;
/*      */   }
/*      */   
/*      */   private Filter[] getFilters(PersistenceMetadata metadata, Map overridenProperties, boolean excludeIfNotOverriden) {
/*  567 */     Properties properties = metadata.getProps();
/*  568 */     List<String> mappingFiles = metadata.getMappingFiles();
/*  569 */     boolean[] detectedArtifacts = getDetectedArtifacts(properties, overridenProperties, excludeIfNotOverriden);
/*      */     
/*  571 */     return getFilters(detectedArtifacts, true, mappingFiles);
/*      */   }
/*      */   
/*      */   private Filter[] getFilters(final boolean[] detectedArtifacts, final boolean searchORM, final List<String> mappingFiles) {
/*  575 */     final int mappingFilesSize = mappingFiles != null ? mappingFiles.size() : 0;
/*  576 */     int size = (detectedArtifacts[0] != 0 ? 2 : 0) + ((searchORM) || (detectedArtifacts[1] != 0) || (mappingFilesSize > 0) ? 1 : 0);
/*  577 */     Filter[] filters = new Filter[size];
/*  578 */     if (detectedArtifacts[0] != 0) {
/*  579 */       filters[0 = new PackageFilter(false, null) {
/*      */         public boolean accept(String javaElementName) {
/*  581 */           return true;
/*      */         }
/*  583 */       };
/*  584 */       filters[1 = new ClassFilter(false, new Class[] { Entity.class, MappedSuperclass.class, Embeddable.class })
/*      */       {
/*      */ 
/*      */ 
/*      */         public boolean accept(String javaElementName)
/*      */         {
/*      */ 
/*  591 */           return true;
/*      */         }
/*      */       };
/*      */     }
/*  595 */     if ((detectedArtifacts[1] != 0) || (searchORM) || (mappingFilesSize > 0)) {
/*  596 */       filters[(size - 1) = new FileFilter(true) {
/*      */         public boolean accept(String javaElementName) {
/*  598 */           return ((detectedArtifacts[1] != 0) && (javaElementName.endsWith("hbm.xml"))) || ((searchORM) && (javaElementName.endsWith("META-INF/orm.xml"))) || ((mappingFilesSize > 0) && (mappingFiles.contains(javaElementName)));
/*      */         }
/*      */       };
/*      */     }
/*      */     
/*      */ 
/*  604 */     return filters;
/*      */   }
/*      */   
/*      */   private void scanForClasses(URL jar, List<String> packages, List<String> entities, List<NamedInputStream> hbmFiles, boolean[] detectedArtifacts, boolean searchORM) {
/*  608 */     if (jar == null) {
/*  609 */       log.error("Container is providing a null PersistenceUnitRootUrl: discovery impossible");
/*  610 */       return;
/*      */     }
/*      */     try {
/*  613 */       JarVisitor visitor = JarVisitorFactory.getVisitor(jar, getFilters(detectedArtifacts, searchORM, null));
/*  614 */       addScannedEntries(visitor, entities, packages, hbmFiles, null);
/*      */     }
/*      */     catch (RuntimeException e) {
/*  617 */       throw new RuntimeException("error trying to scan <jar-file>: " + jar.toString(), e);
/*      */     }
/*      */     catch (IOException e) {
/*  620 */       throw new RuntimeException("Error while reading " + jar.toString(), e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public EntityManagerFactory createEntityManagerFactory(Map workingVars)
/*      */   {
/*  636 */     Properties props = new Properties();
/*  637 */     if (workingVars != null) {
/*  638 */       props.putAll(workingVars);
/*      */       
/*  640 */       props.remove("hibernate.ejb.classes");
/*  641 */       props.remove("hibernate.ejb.packages");
/*  642 */       props.remove("hibernate.hbmxml.files");
/*  643 */       props.remove("hibernate.ejb.loaded.classes");
/*      */     }
/*  645 */     configure(props, workingVars);
/*  646 */     return buildEntityManagerFactory();
/*      */   }
/*      */   
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public EntityManagerFactory createEntityManagerFactory()
/*      */   {
/*  654 */     configure(this.cfg.getProperties(), new HashMap());
/*  655 */     return buildEntityManagerFactory();
/*      */   }
/*      */   
/*      */   public EntityManagerFactory buildEntityManagerFactory() {
/*  659 */     Thread thread = null;
/*  660 */     ClassLoader contextClassLoader = null;
/*  661 */     if (this.overridenClassLoader != null) {
/*  662 */       thread = Thread.currentThread();
/*  663 */       contextClassLoader = thread.getContextClassLoader();
/*  664 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/*  667 */       configure((Properties)null, null);
/*  668 */       NamingHelper.bind(this);
/*  669 */       return new EntityManagerFactoryImpl(this.cfg.buildSessionFactory(), this.transactionType, this.discardOnClose, getSessionInterceptorClass(this.cfg.getProperties()));
/*      */ 
/*      */ 
/*      */     }
/*      */     catch (HibernateException e)
/*      */     {
/*      */ 
/*      */ 
/*  677 */       throw new PersistenceException(getExceptionHeader() + "Unable to build EntityManagerFactory", e);
/*      */     }
/*      */     finally {
/*  680 */       if (thread != null) {
/*  681 */         thread.setContextClassLoader(contextClassLoader);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private Class getSessionInterceptorClass(Properties properties) {
/*  687 */     String sessionInterceptorClassname = (String)properties.get("hibernate.ejb.interceptor.session_scoped");
/*  688 */     if (StringHelper.isNotEmpty(sessionInterceptorClassname)) {
/*      */       try {
/*  690 */         Class interceptorClass = ReflectHelper.classForName(sessionInterceptorClassname, Ejb3Configuration.class);
/*  691 */         interceptorClass.newInstance();
/*  692 */         return interceptorClass;
/*      */       }
/*      */       catch (ClassNotFoundException e) {
/*  695 */         throw new PersistenceException(getExceptionHeader() + "Unable to load " + "hibernate.ejb.interceptor.session_scoped" + ": " + sessionInterceptorClassname, e);
/*      */       }
/*      */       catch (IllegalAccessException e)
/*      */       {
/*  699 */         throw new PersistenceException(getExceptionHeader() + "Unable to instanciate " + "hibernate.ejb.interceptor.session_scoped" + ": " + sessionInterceptorClassname, e);
/*      */       }
/*      */       catch (InstantiationException e)
/*      */       {
/*  703 */         throw new PersistenceException(getExceptionHeader() + "Unable to instanciate " + "hibernate.ejb.interceptor.session_scoped" + ": " + sessionInterceptorClassname, e);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  708 */     return null;
/*      */   }
/*      */   
/*      */   public Reference getReference() throws NamingException
/*      */   {
/*  713 */     log.debug("Returning a Reference to the Ejb3Configuration");
/*  714 */     ByteArrayOutputStream stream = new ByteArrayOutputStream();
/*  715 */     ObjectOutput out = null;
/*      */     byte[] serialized;
/*      */     try {
/*  718 */       out = new ObjectOutputStream(stream);
/*  719 */       out.writeObject(this);
/*  720 */       out.close();
/*  721 */       serialized = stream.toByteArray();
/*  722 */       stream.close();
/*      */     }
/*      */     catch (IOException e) {
/*  725 */       NamingException namingException = new NamingException("Unable to serialize Ejb3Configuration");
/*  726 */       namingException.setRootCause(e);
/*  727 */       throw namingException;
/*      */     }
/*      */     
/*  730 */     return new Reference(Ejb3Configuration.class.getName(), new BinaryRefAddr("object", serialized), Ejb3ConfigurationObjectFactory.class.getName(), null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private Ejb3Configuration configure(Properties properties, Map workingVars)
/*      */   {
/*  746 */     if (this.isConfigurationProcessed) return this;
/*  747 */     this.isConfigurationProcessed = true;
/*  748 */     Properties preparedProperties = prepareProperties(properties, workingVars);
/*  749 */     if (workingVars == null) { workingVars = CollectionHelper.EMPTY_MAP;
/*      */     }
/*  751 */     if (preparedProperties.containsKey("hibernate.ejb.cfgfile")) {
/*  752 */       String cfgFileName = preparedProperties.getProperty("hibernate.ejb.cfgfile");
/*  753 */       this.cfg.configure(cfgFileName);
/*      */     }
/*      */     
/*  756 */     this.cfg.addProperties(preparedProperties);
/*      */     
/*  758 */     addClassesToSessionFactory(workingVars);
/*      */     
/*      */ 
/*  761 */     List<String> jaccKeys = new ArrayList();
/*      */     
/*      */ 
/*  764 */     Interceptor defaultInterceptor = DEFAULT_CONFIGURATION.getInterceptor();
/*  765 */     NamingStrategy defaultNamingStrategy = DEFAULT_CONFIGURATION.getNamingStrategy();
/*      */     
/*  767 */     Iterator propertyIt = preparedProperties.keySet().iterator();
/*  768 */     while (propertyIt.hasNext()) {
/*  769 */       Object uncastObject = propertyIt.next();
/*      */       
/*  771 */       if ((uncastObject != null) && ((uncastObject instanceof String))) {
/*  772 */         String propertyKey = (String)uncastObject;
/*  773 */         if (propertyKey.startsWith("hibernate.ejb.classcache")) {
/*  774 */           setCacheStrategy(propertyKey, preparedProperties, true, workingVars);
/*      */         }
/*  776 */         else if (propertyKey.startsWith("hibernate.ejb.collectioncache")) {
/*  777 */           setCacheStrategy(propertyKey, preparedProperties, false, workingVars);
/*      */         }
/*  779 */         else if ((propertyKey.startsWith("hibernate.jacc")) && (!propertyKey.equals("hibernate.jacc.ctx.id")) && (!propertyKey.equals("hibernate.jacc.enabled")))
/*      */         {
/*      */ 
/*  782 */           jaccKeys.add(propertyKey);
/*      */         }
/*      */       }
/*      */     }
/*  786 */     if ((preparedProperties.containsKey("hibernate.ejb.interceptor")) && ((this.cfg.getInterceptor() == null) || (this.cfg.getInterceptor().equals(defaultInterceptor))))
/*      */     {
/*      */ 
/*      */ 
/*  790 */       String interceptorName = preparedProperties.getProperty("hibernate.ejb.interceptor");
/*      */       try {
/*  792 */         Class interceptor = classForName(interceptorName);
/*  793 */         this.cfg.setInterceptor((Interceptor)interceptor.newInstance());
/*      */       }
/*      */       catch (ClassNotFoundException e) {
/*  796 */         throw new PersistenceException(getExceptionHeader() + "Unable to find interceptor class: " + interceptorName, e);
/*      */ 
/*      */       }
/*      */       catch (IllegalAccessException e)
/*      */       {
/*  801 */         throw new PersistenceException(getExceptionHeader() + "Unable to access interceptor class: " + interceptorName, e);
/*      */ 
/*      */       }
/*      */       catch (InstantiationException e)
/*      */       {
/*  806 */         throw new PersistenceException(getExceptionHeader() + "Unable to instanciate interceptor class: " + interceptorName, e);
/*      */ 
/*      */       }
/*      */       catch (ClassCastException e)
/*      */       {
/*  811 */         throw new PersistenceException(getExceptionHeader() + "Interceptor class does not implement Interceptor interface: " + interceptorName, e);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  816 */     if ((preparedProperties.containsKey("hibernate.ejb.naming_strategy")) && ((this.cfg.getNamingStrategy() == null) || (this.cfg.getNamingStrategy().equals(defaultNamingStrategy))))
/*      */     {
/*      */ 
/*      */ 
/*  820 */       String namingStrategyName = preparedProperties.getProperty("hibernate.ejb.naming_strategy");
/*      */       try {
/*  822 */         Class namingStragegy = classForName(namingStrategyName);
/*  823 */         this.cfg.setNamingStrategy((NamingStrategy)namingStragegy.newInstance());
/*      */       }
/*      */       catch (ClassNotFoundException e) {
/*  826 */         throw new PersistenceException(getExceptionHeader() + "Unable to find naming strategy class: " + namingStrategyName, e);
/*      */ 
/*      */       }
/*      */       catch (IllegalAccessException e)
/*      */       {
/*  831 */         throw new PersistenceException(getExceptionHeader() + "Unable to access naming strategy class: " + namingStrategyName, e);
/*      */ 
/*      */       }
/*      */       catch (InstantiationException e)
/*      */       {
/*  836 */         throw new PersistenceException(getExceptionHeader() + "Unable to instanciate naming strategy class: " + namingStrategyName, e);
/*      */ 
/*      */       }
/*      */       catch (ClassCastException e)
/*      */       {
/*  841 */         throw new PersistenceException(getExceptionHeader() + "Naming strategyy class does not implement NmaingStrategy interface: " + namingStrategyName, e);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  848 */     if (jaccKeys.size() > 0) {
/*  849 */       addSecurity(jaccKeys, preparedProperties, workingVars);
/*      */     }
/*      */     
/*      */ 
/*  853 */     this.listenerConfigurator.setProperties(preparedProperties);
/*  854 */     this.listenerConfigurator.configure();
/*      */     
/*      */ 
/*      */ 
/*  858 */     if (!"true".equalsIgnoreCase(this.cfg.getProperty("hibernate.connection.autocommit"))) {
/*  859 */       log.warn("hibernate.connection.autocommit = false break the EJB3 specification");
/*      */     }
/*  861 */     this.discardOnClose = preparedProperties.getProperty("hibernate.ejb.discard_pc_on_close").equals("true");
/*      */     
/*  863 */     return this;
/*      */   }
/*      */   
/*      */   private void addClassesToSessionFactory(Map workingVars) {
/*  867 */     if (workingVars.containsKey("hibernate.ejb.classes")) {
/*  868 */       java.util.Collection<String> classNames = (java.util.Collection)workingVars.get("hibernate.ejb.classes");
/*      */       
/*      */ 
/*  871 */       addNamedAnnotatedClasses(this, classNames, workingVars);
/*      */     }
/*      */     
/*  874 */     if (workingVars.containsKey("hibernate.ejb.loaded.classes")) {
/*  875 */       java.util.Collection<Class> classes = (java.util.Collection)workingVars.get("hibernate.ejb.loaded.classes");
/*  876 */       for (Class clazz : classes) {
/*  877 */         this.cfg.addAnnotatedClass(clazz);
/*      */       }
/*      */     }
/*  880 */     if (workingVars.containsKey("hibernate.ejb.packages")) {
/*  881 */       java.util.Collection<String> packages = (java.util.Collection)workingVars.get("hibernate.ejb.packages");
/*      */       
/*      */ 
/*  884 */       for (String pkg : packages) {
/*  885 */         this.cfg.addPackage(pkg);
/*      */       }
/*      */     }
/*  888 */     if (workingVars.containsKey("hibernate.ejb.xml_files")) {
/*  889 */       java.util.Collection<String> xmlFiles = (java.util.Collection)workingVars.get("hibernate.ejb.xml_files");
/*      */       
/*      */ 
/*  892 */       for (String xmlFile : xmlFiles) {
/*  893 */         Boolean useMetaInf = null;
/*      */         try {
/*  895 */           if (xmlFile.endsWith("META-INF/orm.xml")) useMetaInf = Boolean.valueOf(true);
/*  896 */           this.cfg.addResource(xmlFile);
/*      */         }
/*      */         catch (MappingNotFoundException e) {
/*  899 */           if (!xmlFile.endsWith("META-INF/orm.xml")) {
/*  900 */             throw new PersistenceException(getExceptionHeader() + "Unable to find XML mapping file in classpath: " + xmlFile);
/*      */           }
/*      */           
/*      */ 
/*  904 */           useMetaInf = Boolean.valueOf(false);
/*      */ 
/*      */         }
/*      */         catch (MappingException me)
/*      */         {
/*  909 */           throw new PersistenceException(getExceptionHeader() + "Error while reading JPA XML file: " + xmlFile, me);
/*      */         }
/*      */         
/*  912 */         if (log.isInfoEnabled()) {
/*  913 */           if (Boolean.TRUE.equals(useMetaInf)) {
/*  914 */             log.info(getExceptionHeader() + "META-INF/orm.xml" + " found");
/*      */           }
/*  916 */           else if (Boolean.FALSE.equals(useMetaInf)) {
/*  917 */             log.info(getExceptionHeader() + "no " + "META-INF/orm.xml" + " found");
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*  922 */     if (workingVars.containsKey("hibernate.hbmxml.files")) {
/*  923 */       java.util.Collection<NamedInputStream> hbmXmlFiles = (java.util.Collection)workingVars.get("hibernate.hbmxml.files");
/*      */       
/*      */ 
/*  926 */       for (NamedInputStream is : hbmXmlFiles) {
/*      */         try
/*      */         {
/*  929 */           this.cfg.addInputStream(new BufferedInputStream(is.getStream()));
/*      */         }
/*      */         catch (MappingException me)
/*      */         {
/*  933 */           if (StringHelper.isEmpty(is.getName())) {
/*  934 */             throw me;
/*      */           }
/*      */           
/*  937 */           throw new MappingException("Error while parsing file: " + is.getName(), me);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private String getExceptionHeader()
/*      */   {
/*  945 */     if (StringHelper.isNotEmpty(this.persistenceUnitName)) {
/*  946 */       return "[PersistenceUnit: " + this.persistenceUnitName + "] ";
/*      */     }
/*      */     
/*  949 */     return "";
/*      */   }
/*      */   
/*      */   private Properties prepareProperties(Properties properties, Map workingVars)
/*      */   {
/*  954 */     Properties preparedProperties = new Properties();
/*      */     
/*      */ 
/*  957 */     preparedProperties.setProperty("hibernate.connection.release_mode", "auto");
/*  958 */     preparedProperties.setProperty("hibernate.query.jpaql_strict_compliance", "true");
/*      */     
/*  960 */     preparedProperties.setProperty("hibernate.connection.autocommit", "true");
/*  961 */     preparedProperties.setProperty("hibernate.use_identifier_rollback", "false");
/*  962 */     preparedProperties.setProperty("hibernate.transaction.flush_before_completion", "false");
/*  963 */     preparedProperties.setProperty("hibernate.ejb.discard_pc_on_close", "false");
/*  964 */     if (this.cfgXmlResource != null) {
/*  965 */       preparedProperties.setProperty("hibernate.ejb.cfgfile", this.cfgXmlResource);
/*  966 */       this.cfgXmlResource = null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  971 */     if (this.cfg.getProperties() != null) { preparedProperties.putAll(this.cfg.getProperties());
/*      */     }
/*  973 */     if (properties != null) { preparedProperties.putAll(properties);
/*      */     }
/*      */     
/*  976 */     if (this.transactionType == null)
/*      */     {
/*  978 */       this.transactionType = PersistenceUnitTransactionType.RESOURCE_LOCAL;
/*      */     }
/*  980 */     defineTransactionType(preparedProperties.getProperty("javax.persistence.transactionType"), workingVars);
/*      */     
/*      */ 
/*      */ 
/*  984 */     boolean hasTxStrategy = StringHelper.isNotEmpty(preparedProperties.getProperty("hibernate.transaction.factory_class"));
/*      */     
/*      */ 
/*  987 */     if ((!hasTxStrategy) && (this.transactionType == PersistenceUnitTransactionType.JTA)) {
/*  988 */       preparedProperties.setProperty("hibernate.transaction.factory_class", JoinableCMTTransactionFactory.class.getName());
/*      */ 
/*      */ 
/*      */     }
/*  992 */     else if ((!hasTxStrategy) && (this.transactionType == PersistenceUnitTransactionType.RESOURCE_LOCAL)) {
/*  993 */       preparedProperties.setProperty("hibernate.transaction.factory_class", JDBCTransactionFactory.class.getName());
/*      */     }
/*  995 */     if (hasTxStrategy) {
/*  996 */       log.warn("Overriding hibernate.transaction.factory_class is dangerous, this might break the EJB3 specification implementation");
/*      */     }
/*      */     
/*      */ 
/* 1000 */     if (preparedProperties.getProperty("hibernate.transaction.flush_before_completion").equals("true")) {
/* 1001 */       preparedProperties.setProperty("hibernate.transaction.flush_before_completion", "false");
/* 1002 */       log.warn("Defining hibernate.transaction.flush_before_completion=true ignored in HEM");
/*      */     }
/* 1004 */     return preparedProperties;
/*      */   }
/*      */   
/*      */   private Class classForName(String className) throws ClassNotFoundException {
/* 1008 */     return ReflectHelper.classForName(className, getClass());
/*      */   }
/*      */   
/*      */   private void setCacheStrategy(String propertyKey, Map properties, boolean isClass, Map workingVars) {
/* 1012 */     String role = propertyKey.substring((isClass ? "hibernate.ejb.classcache".length() : "hibernate.ejb.collectioncache".length()) + 1);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1018 */     String value = (String)properties.get(propertyKey);
/* 1019 */     StringTokenizer params = new StringTokenizer(value, ";, ");
/* 1020 */     if (!params.hasMoreTokens()) {
/* 1021 */       StringBuilder error = new StringBuilder("Illegal usage of ");
/* 1022 */       error.append(isClass ? "hibernate.ejb.classcache" : "hibernate.ejb.collectioncache");
/*      */       
/*      */ 
/* 1025 */       error.append(": ").append(propertyKey).append(" ").append(value);
/* 1026 */       throw new PersistenceException(getExceptionHeader() + error.toString());
/*      */     }
/* 1028 */     String usage = params.nextToken();
/* 1029 */     String region = null;
/* 1030 */     if (params.hasMoreTokens()) {
/* 1031 */       region = params.nextToken();
/*      */     }
/* 1033 */     if (isClass) {
/* 1034 */       boolean lazyProperty = true;
/* 1035 */       if (params.hasMoreTokens()) {
/* 1036 */         lazyProperty = "all".equalsIgnoreCase(params.nextToken());
/*      */       }
/* 1038 */       this.cfg.setCacheConcurrencyStrategy(role, usage, region, lazyProperty);
/*      */     }
/*      */     else {
/* 1041 */       this.cfg.setCollectionCacheConcurrencyStrategy(role, usage, region);
/*      */     }
/*      */   }
/*      */   
/*      */   private void addSecurity(List<String> keys, Map properties, Map workingVars) {
/* 1046 */     log.debug("Adding security");
/* 1047 */     if (!properties.containsKey("hibernate.jacc.ctx.id")) {
/* 1048 */       throw new PersistenceException(getExceptionHeader() + "Entities have been configured for JACC, but " + "hibernate.jacc.ctx.id" + " has not been set");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1054 */     String contextId = (String)properties.get("hibernate.jacc.ctx.id");
/* 1055 */     setProperty("hibernate.jacc_context_id", contextId);
/*      */     
/* 1057 */     int roleStart = "hibernate.jacc".length() + 1;
/*      */     
/* 1059 */     for (String key : keys) {
/* 1060 */       JACCConfiguration jaccCfg = new JACCConfiguration(contextId);
/*      */       try {
/* 1062 */         String role = key.substring(roleStart, key.indexOf('.', roleStart));
/* 1063 */         int classStart = roleStart + role.length() + 1;
/* 1064 */         String clazz = key.substring(classStart, key.length());
/* 1065 */         String actions = (String)properties.get(key);
/* 1066 */         jaccCfg.addPermission(role, clazz, actions);
/*      */       }
/*      */       catch (IndexOutOfBoundsException e) {
/* 1069 */         throw new PersistenceException(getExceptionHeader() + "Illegal usage of " + "hibernate.jacc" + ": " + key);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void addNamedAnnotatedClasses(Ejb3Configuration cfg, java.util.Collection<String> classNames, Map workingVars)
/*      */   {
/* 1078 */     for (String name : classNames) {
/*      */       try {
/* 1080 */         Class clazz = classForName(name);
/* 1081 */         cfg.addAnnotatedClass(clazz);
/*      */       }
/*      */       catch (ClassNotFoundException cnfe) {
/*      */         Package pkg;
/*      */         try {
/* 1086 */           pkg = classForName(name + ".package-info").getPackage();
/*      */         }
/*      */         catch (ClassNotFoundException e) {
/* 1089 */           pkg = null;
/*      */         }
/* 1091 */         if (pkg == null) {
/* 1092 */           throw new PersistenceException(getExceptionHeader() + "class or package not found", cnfe);
/*      */         }
/*      */         
/* 1095 */         cfg.addPackage(name);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public Settings buildSettings()
/*      */     throws HibernateException
/*      */   {
/* 1103 */     Thread thread = null;
/* 1104 */     ClassLoader contextClassLoader = null;
/* 1105 */     if (this.overridenClassLoader != null) {
/* 1106 */       thread = Thread.currentThread();
/* 1107 */       contextClassLoader = thread.getContextClassLoader();
/* 1108 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1111 */       return this.settingsFactory.buildSettings(this.cfg.getProperties());
/*      */     }
/*      */     finally {
/* 1114 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addProperties(Properties props) {
/* 1119 */     this.cfg.addProperties(props);
/* 1120 */     return this;
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addAnnotatedClass(Class persistentClass) throws MappingException {
/* 1124 */     Thread thread = null;
/* 1125 */     ClassLoader contextClassLoader = null;
/* 1126 */     if (this.overridenClassLoader != null) {
/* 1127 */       thread = Thread.currentThread();
/* 1128 */       contextClassLoader = thread.getContextClassLoader();
/* 1129 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1132 */       this.cfg.addAnnotatedClass(persistentClass);
/* 1133 */       return this;
/*      */     }
/*      */     finally {
/* 1136 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration configure(String resource) throws HibernateException
/*      */   {
/* 1142 */     if (this.cfgXmlResource != null)
/* 1143 */       throw new PersistenceException("configure(String) method already called for " + this.cfgXmlResource);
/* 1144 */     this.cfgXmlResource = resource;
/* 1145 */     return this;
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addPackage(String packageName) throws MappingException {
/* 1149 */     Thread thread = null;
/* 1150 */     ClassLoader contextClassLoader = null;
/* 1151 */     if (this.overridenClassLoader != null) {
/* 1152 */       thread = Thread.currentThread();
/* 1153 */       contextClassLoader = thread.getContextClassLoader();
/* 1154 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1157 */       this.cfg.addPackage(packageName);
/* 1158 */       return this;
/*      */     }
/*      */     finally {
/* 1161 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addFile(String xmlFile) throws MappingException {
/* 1166 */     Thread thread = null;
/* 1167 */     ClassLoader contextClassLoader = null;
/* 1168 */     if (this.overridenClassLoader != null) {
/* 1169 */       thread = Thread.currentThread();
/* 1170 */       contextClassLoader = thread.getContextClassLoader();
/* 1171 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1174 */       this.cfg.addFile(xmlFile);
/* 1175 */       return this;
/*      */     }
/*      */     finally {
/* 1178 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addClass(Class persistentClass) throws MappingException {
/* 1183 */     Thread thread = null;
/* 1184 */     ClassLoader contextClassLoader = null;
/* 1185 */     if (this.overridenClassLoader != null) {
/* 1186 */       thread = Thread.currentThread();
/* 1187 */       contextClassLoader = thread.getContextClassLoader();
/* 1188 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1191 */       this.cfg.addClass(persistentClass);
/* 1192 */       return this;
/*      */     }
/*      */     finally {
/* 1195 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addFile(File xmlFile) throws MappingException {
/* 1200 */     Thread thread = null;
/* 1201 */     ClassLoader contextClassLoader = null;
/* 1202 */     if (this.overridenClassLoader != null) {
/* 1203 */       thread = Thread.currentThread();
/* 1204 */       contextClassLoader = thread.getContextClassLoader();
/* 1205 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1208 */       this.cfg.addFile(xmlFile);
/* 1209 */       return this;
/*      */     }
/*      */     finally {
/* 1212 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public void buildMappings() {
/* 1217 */     Thread thread = null;
/* 1218 */     ClassLoader contextClassLoader = null;
/* 1219 */     if (this.overridenClassLoader != null) {
/* 1220 */       thread = Thread.currentThread();
/* 1221 */       contextClassLoader = thread.getContextClassLoader();
/* 1222 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1225 */       this.cfg.buildMappings();
/*      */     }
/*      */     finally {
/* 1228 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Iterator getClassMappings() {
/* 1233 */     Thread thread = null;
/* 1234 */     ClassLoader contextClassLoader = null;
/* 1235 */     if (this.overridenClassLoader != null) {
/* 1236 */       thread = Thread.currentThread();
/* 1237 */       contextClassLoader = thread.getContextClassLoader();
/* 1238 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1241 */       return this.cfg.getClassMappings();
/*      */     }
/*      */     finally {
/* 1244 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public EventListeners getEventListeners() {
/* 1249 */     return this.cfg.getEventListeners();
/*      */   }
/*      */   
/*      */   SessionFactory buildSessionFactory() throws HibernateException {
/* 1253 */     return this.cfg.buildSessionFactory();
/*      */   }
/*      */   
/*      */   public Iterator getTableMappings() {
/* 1257 */     return this.cfg.getTableMappings();
/*      */   }
/*      */   
/*      */   public PersistentClass getClassMapping(String persistentClass) {
/* 1261 */     return this.cfg.getClassMapping(persistentClass);
/*      */   }
/*      */   
/*      */   public org.hibernate.mapping.Collection getCollectionMapping(String role) {
/* 1265 */     return this.cfg.getCollectionMapping(role);
/*      */   }
/*      */   
/*      */   public void setEntityResolver(EntityResolver entityResolver) {
/* 1269 */     this.cfg.setEntityResolver(entityResolver);
/*      */   }
/*      */   
/*      */   public Map getNamedQueries() {
/* 1273 */     return this.cfg.getNamedQueries();
/*      */   }
/*      */   
/*      */   public Interceptor getInterceptor() {
/* 1277 */     return this.cfg.getInterceptor();
/*      */   }
/*      */   
/*      */   public Properties getProperties() {
/* 1281 */     return this.cfg.getProperties();
/*      */   }
/*      */   
/*      */   public Ejb3Configuration setInterceptor(Interceptor interceptor) {
/* 1285 */     this.cfg.setInterceptor(interceptor);
/* 1286 */     return this;
/*      */   }
/*      */   
/*      */   public Ejb3Configuration setProperties(Properties properties) {
/* 1290 */     this.cfg.setProperties(properties);
/* 1291 */     return this;
/*      */   }
/*      */   
/*      */   public Map getFilterDefinitions() {
/* 1295 */     return this.cfg.getFilterDefinitions();
/*      */   }
/*      */   
/*      */   public void addFilterDefinition(FilterDefinition definition) {
/* 1299 */     this.cfg.addFilterDefinition(definition);
/*      */   }
/*      */   
/*      */   public void addAuxiliaryDatabaseObject(AuxiliaryDatabaseObject object) {
/* 1303 */     this.cfg.addAuxiliaryDatabaseObject(object);
/*      */   }
/*      */   
/*      */   public NamingStrategy getNamingStrategy() {
/* 1307 */     return this.cfg.getNamingStrategy();
/*      */   }
/*      */   
/*      */   public Ejb3Configuration setNamingStrategy(NamingStrategy namingStrategy) {
/* 1311 */     this.cfg.setNamingStrategy(namingStrategy);
/* 1312 */     return this;
/*      */   }
/*      */   
/*      */   public void setListeners(String type, String[] listenerClasses) {
/* 1316 */     this.cfg.setListeners(type, listenerClasses);
/*      */   }
/*      */   
/*      */   public void setListeners(String type, Object[] listeners) {
/* 1320 */     this.cfg.setListeners(type, listeners);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public AnnotationConfiguration getHibernateConfiguration()
/*      */   {
/* 1331 */     return this.cfg;
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addInputStream(InputStream xmlInputStream) throws MappingException {
/* 1335 */     Thread thread = null;
/* 1336 */     ClassLoader contextClassLoader = null;
/* 1337 */     if (this.overridenClassLoader != null) {
/* 1338 */       thread = Thread.currentThread();
/* 1339 */       contextClassLoader = thread.getContextClassLoader();
/* 1340 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1343 */       this.cfg.addInputStream(xmlInputStream);
/* 1344 */       return this;
/*      */     }
/*      */     finally {
/* 1347 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addResource(String path) throws MappingException {
/* 1352 */     Thread thread = null;
/* 1353 */     ClassLoader contextClassLoader = null;
/* 1354 */     if (this.overridenClassLoader != null) {
/* 1355 */       thread = Thread.currentThread();
/* 1356 */       contextClassLoader = thread.getContextClassLoader();
/* 1357 */       thread.setContextClassLoader(this.overridenClassLoader);
/*      */     }
/*      */     try {
/* 1360 */       this.cfg.addResource(path);
/* 1361 */       return this;
/*      */     }
/*      */     finally {
/* 1364 */       if (thread != null) thread.setContextClassLoader(contextClassLoader);
/*      */     }
/*      */   }
/*      */   
/*      */   public Ejb3Configuration addResource(String path, ClassLoader classLoader) throws MappingException {
/* 1369 */     this.cfg.addResource(path, classLoader);
/* 1370 */     return this;
/*      */   }
/*      */   
/*      */   private static enum XML_SEARCH {
/* 1374 */     HBM, 
/* 1375 */     ORM_XML, 
/* 1376 */     BOTH, 
/* 1377 */     NONE;
/*      */     
/*      */     private XML_SEARCH() {}
/* 1380 */     public static XML_SEARCH getType(boolean searchHbm, boolean searchOrm) { return searchOrm ? ORM_XML : searchHbm ? HBM : searchOrm ? BOTH : NONE; }
/*      */   }
/*      */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-entitymanager-3.3.2.GA.jar!\org\hibernate\ejb\Ejb3Configuration.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */